@@ -6,6 +6,19 @@ from scipy.integrate import simps
 from skimage.transform import rotate
 
 def imrotate_new(img, theta):
+    """
+    Rotation function.
+
+    Parameters
+    ----------
+    img     : image
+    theta   : angle of rotation
+
+    Returns
+    -------
+    Ra      : rotated image
+    
+    """
     n    = img.shape[0]
     th   = np.deg2rad(theta)
     x_1n = np.arange(n)
@@ -25,23 +38,27 @@ def imrotate_new(img, theta):
     return Ra
     
 def my_radon(M, theta):
-    '''
-    M = image
-    thetaval = list of angles
-    
-    TO DO:
-    volendo si puo' ruotare la griglia per non usare imrotate
-    '''
+    """
+    Radon Transform.
+
+    Parameters
+    ----------
+    M        : image
+    thetaval : list of angles
+
+    Returns
+    -------
+    Rt       : Radon transform
+    """
     m,n = M.shape
     theta_len = len(theta)
     R = np.zeros((m,theta_len))
 
     x, y = np.meshgrid(np.linspace(0,1,m), np.linspace(0,1,n))
     
-    
     for i in range(theta_len):
         R_theta = rotate(M, 90 - theta[i])
         #R[ :, i] = np.sum(R_theta, axis = 1)
         R[ :,i] = simps( y = R_theta, x = x, axis = 1) 
-
-    return R[::-1,:]
+    Rt = R[::-1,:]
+    return Rt
