@@ -9,71 +9,62 @@ def my_phantomgallery( phantom_type ):
 
     Parameters
     ----------
-    phantom_type: 'ellipses', 'shepp_logan','modified_shepp_logan','squares','rectangles'
-    
+    phantom_type: 'ellipses' (or 'shepp_logan'),'modified_shepp_logan','squares','rectangles'
+
     Returns
     -------
     M : matrix of the elements of the phantom
     """
-    
-    if phantom_type == 'ellipses':
-        # [semiaxis 1, semiaxis 2, x center, y center, phi=angle, greyscale=attenuation]
+
+    if phantom_type == 'ellipses' or phantom_type == 'shepp_logan':
+        # [semiaxis 1, semiaxis 2, x center, y center, phi=angle (degrees), greyscale=attenuation]
+        M =  np.array([[  .69,   .92,     0,      0,   0,   1.],
+                       [ .6624, .8740,    0, -.0184,   0, -0.8],
+                       [ .1100, .3100,  .22,      0, -18, -.2],
+                       [ .1600, .4100, -.22,      0, 18, -.2],
+                       [ .2100, .2500,    0,    .35,   0, .1],
+                       [ .0460, .0460,    0,     .1,   0, .1],
+                       [ .0460, .0460,    0,    -.1,   0, .1],
+                       [ .0460, .0230, -.08,  -.605,   0, .1],
+                       [ .0230, .0230,    0,  -.605,   0, .1],
+                       [ .0230, .0460,  .06,  -.605,   0, .1]])
+
+
+    elif phantom_type == 'modified_shepp_logan':
+        # [semiaxis 1, semiaxis 2, x center, y center, phi=angle (degrees), greyscale=attenuation]
         p1 = [.7, .8, 0, 0, 0, 1]
         p2 = [.65,.75,0,0,0,-.9]
         p3 = [.15,.2,0,.4,0,.5]
-        p4 = [.25,.15,-.25,.25,2.37,.2]
-        p5 = [.25,.15,.25,.25,.79,.2]
-        p6 = [.08,.25,0,-.3,.5,.65]
+        p4 = [.25,.15,-.25,.25,135.79,.2]
+        p5 = [.25,.15,.25,.25,45.26,.2]
+        p6 = [.08,.25,0,-.3,28.65,.65]
         p7 = [.05,.05,.5,-.3,0,.8]
         # combine into a matrix with one ellipse in each row
         M = np.array([p1, p2, p3, p4, p5, p6, p7]);
+        
 
-    elif phantom_type == 'shepp_logan':
-        # [semiaxis 1, semiaxis 2, x center, y center, phi=angle, greyscale=attenuation]
-        M = np.array( [[.69,   .92,    0,     0,    0, 2   ],
-                [ .6624, .8740,    0, -.0184,   0, -.98],
-                [ .1100, .3100,  .22,      0, -18,-.02],
-                [ .1600, .4100, -.22,      0,  18,-.02],
-                [ .2100, .2500,    0,    .35,   0,.01],
-                [ .0460, .0460,    0,     .1,   0,.01],
-                [ .0460, .0460,    0,    -.1,   0,.02],
-                [ .0460, .0230, -.08,  -.605,   0,.01],
-                [ .0230, .0230,    0,  -.606,   0,.01],
-                [ .0230, .0460,  .06,  -.605,   0,.01]])
-
-    elif phantom_type == 'modified_shepp_logan':
-        # [semiaxis 1, semiaxis 2, x center, y center, phi=angle, greyscale=attenuation]
-        M =  np.array([[  .69,   .92,    0,      0,   0, 1.],
-            [ .6624, .8740,    0, -.0184,   0, -.8],
-            [ .1100, .3100,  .22,      0, -18, -.2],
-            [ .1600, .4100, -.22,      0,  18, -.2],
-            [ .2100, .2500,    0,    .35,   0, .1],
-            [ .0460, .0460,    0,     .1,   0, .1],
-            [ .0460, .0460,    0,    -.1,   0, .1],
-            [ .0460, .0230, -.08,  -.605,   0, .1],
-            [ .0230, .0230,    0,  -.606,   0, .1],
-            [ .0230, .0460,  .06,  -.605,   0, .1]])
 
     elif phantom_type == 'squares':
-        #if 0 each square is a 5-D vector [x0,y0,w,phi=angle,greyscale=attenuation]
+        # [x center, y center, edge length ,phi=angle (degrees), greyscale=attenuation]
         s1 = [0,0,1.3,0,1]
         s2 = [0,0,1.1,0,-.9]
-        s3 = [.1,-.1,.5,np.pi/6,.4]
-        s4 = [-.25,.15,.25,np.pi/4,.2]
-        s5 = [-.2,.25,.3,np.pi/3,.4]
+        s3 = [.1,-.1,.5,180/6,.4]
+        s4 = [-.25,.15,.25,180/4,.2]
+        s5 = [-.2,.25,.3,180/3,.4]
         #combine into a matrix with one square in each row
         M = np.array([s1, s2, s3, s4, s5]);
 
     elif (phantom_type == 'rectangles'):
-        # [x center, y center, dimension 1, dimension 2, phi=angle, greyscale=attenuation]
+        # [x center, y center, dimension 1, dimension 2, phi=angle (degrees), greyscale=attenuation]
         r1 = [0,0,1.3,1.1,0,1]
         r2 = [0,0,1.2,1,0,-.9]
-        r3 = [0.25,.15,.25,.6,np.pi/6,.4]
-        r4 = [-.2,.1,.25,.20,np.pi/4,.2]
-        r5 = [-.3,.2,.3,.2,np.pi/6,.4]
+        r3 = [0.25,.15,.25,.6,180/6,.4]
+        r4 = [-.2,.1,.25,.20,180/4,.2]
+        r5 = [-.3,.2,.3,.2,180/6,.4]
         #combine into a matrix with one square in each row
         M = np.array([r1, r2, r3, r4, r5])
     else:
+        print('Unknown phantom_type')
         M = None
 
     return M
@@ -87,31 +78,43 @@ def phantom_ellipses(n_points,E):
     ----------
     n_points:  number of pixels on each row and column
     E:         matrix of the elements of the phantom
-    
+
     Returns
     -------
     phantom : phantom image
     """
-    
-    x,y = np.meshgrid(np.arange(-1,1,2./n_points),np.arange(-1,1,2./n_points))
+      
+    #Rescaling according to image size 
+    E[:,0] = E[:,0]*n_points/2 #semiaxis a
+    E[:,1] = E[:,1]*n_points/2 #semiaxis b
+    E[:,2] = E[:,2]*n_points/2 #x
+    E[:,3] = E[:,3]*n_points/2 #y
+    E[:,4] = E[:,4]*math.pi/180
+   
+    x,y = np.meshgrid(np.arange(0,n_points)-n_points//2 ,np.arange(0,n_points)-n_points//2 )
     nrow,ncol = E.shape
-
     phantom1 = np.zeros((y.shape[0], y.shape[1], nrow))
 
     for k in range(nrow): #itero sulle ellissi
         x_new = x - E[k,2]
         y_new = y - E[k,3]
 
-
-        cond = (E[k,1]**2)*np.square(x_new * math.cos(E[k,4]) + y_new * math.sin(E[k,4])) + \
-               (E[k,0]**2)*np.square(y_new * math.cos(E[k,4]) - x_new * math.sin(E[k,4])) - (E[k,0]*E[k,1])**2
+        #find(( (x.*cosp + y.*sinp).^2)./asq + ((y.*cosp - x.*sinp).^2)./bsq <= 1); 
+        cosp = math.cos(E[k,4])
+        sinp = math.sin(E[k,4])
+        cond = np.square( x_new * cosp + y_new * sinp )*1/(E[k,0]*E[k,0]) + \
+               np.square(y_new * cosp - x_new * sinp)*1/(E[k,1]*E[k,1]) - 1
 
         for i in range(x.shape[0]):
             for j in range(x.shape[1]):
-                if (cond[i,j] < 0.0):
+                if (cond[i,j] <= 0.0):
                     phantom1[i,j,k] = E[k,5];   # gray scale
                 else:
                     phantom1[i,j,k] = 0.0;
+                #endif
+            #endfor
+         #endfor
+    #endfor
     phantom1 = phantom1.sum(axis=2)
     phantom = np.flipud(phantom1)
     return phantom
@@ -124,14 +127,21 @@ def phantom_squares(n_points,S):
     ----------
     n_points:  number of pixels on each row and column
     S:         matrix of the elements of the phantom
-    
+
     Returns
     -------
     phantom : phantom image
     """
-    x,y = np.meshgrid(np.arange(-1,1,2/n_points),np.arange(-1,1,2/n_points))    #ogni coppia (x,y) identifica un pixel (coordinate del suo centro?)
+    
+    #Rescaling according to image size 
+    S[:,0] = S[:,0]*n_points/2
+    S[:,1] = S[:,1]*n_points/2
+    S[:,2] = S[:,2]*n_points/2
+    S[:,3] = S[:,3]*math.pi/180
+    
+    x,y = np.meshgrid(np.arange(0,n_points)-n_points//2 ,np.arange(0,n_points)-n_points//2 )  
     nrow,ncol = S.shape
-    phantom1 = np.zeros((y.shape[0], y.shape[1], nrow))                         #tensore 3D (la terza dimensione individua i quadrati)
+    phantom1 = np.zeros((y.shape[0], y.shape[1], nrow))                        
 
     for k in range(nrow): #itero sui quadrati
         x_new = x - S[k,0]
@@ -165,12 +175,21 @@ def phantom_rectangles(n_points,R):
     ----------
     n_points:  number of pixels on each row and column
     R:         matrix of the elements of the phantom
-    
+
     Returns
     -------
     phantom : phantom image
     """
-    x,y = np.meshgrid(np.arange(-1,1,2/n_points),np.arange(-1,1,2/n_points))
+    
+    
+    #Rescaling according to image size 
+    R[:,0] = R[:,0]*n_points/2
+    R[:,1] = R[:,1]*n_points/2
+    R[:,2] = R[:,2]*n_points/2
+    R[:,3] = R[:,3]*n_points/2
+    R[:,4] = R[:,4]*math.pi/180
+    
+    x,y = np.meshgrid(np.arange(0,n_points)-n_points//2 ,np.arange(0,n_points)-n_points//2 )
     nrow,ncol = R.shape
     phantom1 = np.zeros((y.shape[0], y.shape[1], nrow))
 
@@ -196,25 +215,26 @@ def phantom_rectangles(n_points,R):
     phantom = np.flipud(phantom1)
     return phantom
 
-def my_radon_analytic(phantom_type, N, theta_vec, M ,  tval_set=None, circle=False ):
+
+def my_radon_analytic(phantom_type, N, theta_vec, M ,  tvec_set=None, circle=False ):
     """
     Function that returns the analytical_sinogram given the phantom.
 
     Parameters
     ----------
     phantom_type : type of the phantom ('ellipses', 'shepp_logan', 'modified_shepp_logan','squares'rectangles')
-    theta_vec     : list of the angles
+    theta_vec    : list of the angles
     M            : matrix of the structure of the phantom
-    tval_set     : 
-    circle       : 
-    
+    tvec_set     : vector of the t values given from by the user
+    circle       : as in the function iradon of scikit-image "assume the reconstructed image is zero outside the inscribed circle. Also changes the default output_size to match the behaviour of radon called with circle=True."
+
     Returns
     -------
     analytical_sinogram : Analytical Sinogram of the given phantom
     """
-    
+
     if phantom_type in ['ellipses', 'shepp_logan', 'modified_shepp_logan']:
-            analytical_sinogram = radon_ellipses(N,theta_vec,M, tval_set,circle);
+            analytical_sinogram = radon_ellipses(N,theta_vec,M, tvec_set,circle);
     elif phantom_type== 'squares':
             analytical_sinogram = radon_squares(N,theta_vec,M, circle);
     elif phantom_type== 'rectangles':
@@ -222,71 +242,87 @@ def my_radon_analytic(phantom_type, N, theta_vec, M ,  tval_set=None, circle=Fal
     else:
         print('error on the choice of phantom type')
     #endif
+    
     return analytical_sinogram
 
 
 
-def radon_ellipses(N,theta_vec, E, tval_set=None, circle=False):
+def radon_ellipses(N,theta_vec, E, tvec_set=None, circle=False):
     """
-    Function that compute the analytical_sinogram for phantoms of ellipses type 
+    Function that compute the analytical_sinogram for phantoms of ellipses type
 
     Parameters
     ----------
-    N        : dimension of the image
+    N         : Pixels per dimension of the image
     theta_vec : vector of the angles theta
-    E        : matrix of the ellipses parameters
-    tval_set : 
-    circle   : 
-    
+    E         : matrix of the ellipses parameters
+    tvec_set  : vector of the t values given from by the user
+    circle    : as in the function iradon of scikit-image "assume the reconstructed image is zero outside the inscribed circle. Also changes the default output_size to match the behaviour of radon called with circle=True."
+
     Returns
     -------
     analytical_sinogram : Analytical Sinogram
     """
-
-
-    [t_vec, grid_t, grid_theta] = build_t_theta(N,theta_vec, tval_set=tval_set, circle =circle);
+    
+    #Rescaling according to image size 
+    E[:,0] = E[:,0]*N/2
+    E[:,1] = E[:,1]*N/2
+    E[:,2] = E[:,2]*N/2
+    E[:,3] = E[:,3]*N/2
+    E[:,4] = E[:,4]*math.pi/180
+    
+    [t_vec, grid_t, grid_theta] = build_t_theta_pixel(N, theta_vec, tvec_set=tvec_set, circle =circle);
 
     (nrowE,ncolE) = E.shape;
-    tmp = np.zeros((nrowE,len(grid_theta)));
+    tmp = np.zeros((nrowE,len(grid_theta)))
     for i in range(nrowE):
-        grid_theta_new = grid_theta - E[i,4];
-        grid_t_new     = (grid_t - E[i,2]*np.cos(grid_theta)-E[i,3]*np.sin(grid_theta))/E[i,1];
+        grid_theta_new = grid_theta - E[i,4]
+        x_new = (E[i,2]*np.cos(grid_theta)+E[i,3]*np.sin(grid_theta))
+        y_new = (-E[i,2]*np.sin(grid_theta)+E[i,3]*np.cos(grid_theta))
+        grid_t_new = (grid_t -x_new)/E[i,1]
 
-        v1 = np.sin(grid_theta_new)**2+((E[i,0]/E[i,1])**2)*np.cos(grid_theta_new)**2 - grid_t_new**2;
+        v1 = np.sin(grid_theta_new)**2+((E[i,0]/E[i,1])**2)*np.cos(grid_theta_new)**2 - grid_t_new**2
         cond = v1;
-        v2 = np.zeros((v1.shape[0],1));
+        v2 = np.zeros((v1.shape[0],1))
         for j in range (len(grid_theta)):
             if cond[j] > 0:
-                v2[j]=1;
+                v2[j]=1
             else:
-                v2[j]=0;
+                v2[j]=0
             #endif
         #endfor
         v3 = np.sqrt(v1*v2);
-        v4 = np.sin(grid_theta_new)**2+((E[i,0]/E[i,1])**2)*np.cos(grid_theta_new)**2;
-        tmp[i,:] = np.transpose( E[i,0]*E[i,5]*(v3/v4) );
+        v4 = np.sin(grid_theta_new)**2+((E[i,0]/E[i,1])**2)*np.cos(grid_theta_new)**2
+        tmp[i,:] = np.transpose( 2*E[i,0]*E[i,5]*(v3/v4) )
     #endfor
     radvec = np.sum(tmp,axis = 0);
-    analytical_sinogram = np.transpose(np.reshape(radvec,(len(theta_vec),len(t_vec))));
+    analytical_sinogram = np.transpose(np.reshape(radvec,(len(theta_vec),len(t_vec))))
     return  analytical_sinogram
 
 
 def radon_squares(N,theta_vec,S, circle=False):
     """
-    Function that compute the analytical_sinogram for phantoms of square type 
+    Function that compute the analytical_sinogram for phantoms of square type
 
     Parameters
     ----------
-    N        : dimension of the image
+    N         : dimension of the image
     theta_vec : list of the angles
-    S        : matrix of the squares parameters
-    circle   : 
-    
+    S         : matrix of the squares parameters
+    circle    : as in the function iradon of scikit-image "assume the reconstructed image is zero outside the inscribed circle. Also changes the default output_size to match the behaviour of radon called with circle=True."
+
     Returns
     -------
     analytical_sinogram : Analytical Sinogram
     """
-    [t_vec, grid_t, grid_theta] = build_t_theta(N,theta_vec, circle = circle);
+    
+    #Rescaling according to image size 
+    S[:,0] = S[:,0]*N/2
+    S[:,1] = (S[:,1])*N/2
+    S[:,2] = (S[:,2])*N/2
+    S[:,3] = S[:,3]*math.pi/180
+    
+    [t_vec, grid_t, grid_theta] = build_t_theta_pixel(N,theta_vec, circle = circle);
     [nrow,ncol] = np.shape(S);
     tmp = np.zeros((nrow,len(grid_theta)));
     for i in range(nrow):       # cycle on the elements of the phantom
@@ -336,10 +372,9 @@ def radon_squares(N,theta_vec,S, circle=False):
         #endfor
     #endfor
     radvec = np.sum(tmp,axis=0);
-    radvec = radvec/np.amax(radvec);
     
     analytical_sinogram = np.transpose(np.reshape(radvec,(len(theta_vec),len(t_vec))));
-    
+
     return  analytical_sinogram
 
 
@@ -347,23 +382,31 @@ def radon_squares(N,theta_vec,S, circle=False):
 
 def radon_rectangles(N,theta_vec,R, circle = False):
     """
-    Function that compute the analytical_sinogram for phantoms of rectangle type 
+    Function that compute the analytical_sinogram for phantoms of rectangle type
 
     Parameters
     ----------
     N        : dimension of the image
-    theta_vec : list of the angles
+    theta_vec: list of the angles
     R        : matrix of the rectangle parameters
-    circle   : 
-    
+    circle   : as in the function iradon of scikit-image "assume the reconstructed image is zero outside the inscribed circle. Also changes the default output_size to match the behaviour of radon called with circle=True."
+
     Returns
     -------
     analytical_sinogram : Analytical Sinogram
     """
-    [t_vec, grid_t, grid_theta] = build_t_theta(N,theta_vec, circle=circle);
-
+    
+    #Rescaling according to image size 
+    R[:,0] = R[:,0]*N/2
+    R[:,1] = (R[:,1])*N/2
+    R[:,2] = (R[:,2])*N/2
+    R[:,3] = (R[:,3])*N/2
+    R[:,4] = R[:,4]*math.pi/180
+    
+    [t_vec, grid_t, grid_theta] = build_t_theta_pixel(N,theta_vec, circle=circle);
     (nrow, ncol) = R.shape;
-    tmp = np.zeros((nrow,len(grid_theta)));
+    tmp = np.zeros((nrow,len(grid_theta)))
+    
     for i in range(nrow):
         m = R[i,2]/2;
         n = R[i,3]/2;
@@ -379,7 +422,7 @@ def radon_rectangles(N,theta_vec,R, circle = False):
                 else:
                     v1 = 0;
                     v2 = 0;
-				    #endif
+                #endif
             else:
                 v1 = (t_new*np.cos(theta_new)- m)/np.sin(theta_new);
                 v2 = (t_new*np.cos(theta_new)+ m)/np.sin(theta_new);
@@ -411,59 +454,52 @@ def radon_rectangles(N,theta_vec,R, circle = False):
         #endfor
     #endfor
     radvec = np.sum(tmp,axis = 0);
-    radvec = radvec/np.amax(radvec);
     analytical_sinogram = np.transpose(np.reshape(radvec,(len(theta_vec),len(t_vec))));
-    
+
     return  analytical_sinogram
-#end
 
 
 
-def build_t_theta(N,theta_vec, tval_set=None, circle=False):
+def build_t_theta_pixel(N,theta_vec, tvec_set=None, circle=False):
     """
     Function that compute the grid of t and theta for the radon trasform
 
     Parameters
     ----------
     N        : dimension of the image
-    theta_vec : list of the angles
-    tval_set : 
-    circle   : 
-    
+    theta_vec: list of the angles
+    tvec_set : vector of the t values given from by the user
+    circle   : as in the function iradon of scikit-image "assume the reconstructed image is zero outside the inscribed circle. Also changes the default output_size to match the behaviour of radon called with circle=True."
+
     Returns
     -------
-    t_vec       : vector of the t values 
+    t_vec       : vector of the t values
     grid_t      : grid of the t values
     grid_theta  : grid of the theta values
     """
 
     Nangle = len(theta_vec)
 
-    if tval_set is None:
-        print('t_vec default')
-        dt = 2./(N-1)
+    if tvec_set is None:
         if not circle:
             N = int(np.ceil(N*np.sqrt(2)))
-            t_vec = np.sqrt(2) *np.linspace(-1+dt,1-dt,N)
+            t_vec = np.sqrt(2) * np.arange(-N/2+0.5, N/2 )  
         else:
-            t_vec = np.linspace(-1+dt,1-dt,N)
+            t_vec = np.arange(-N/2+0.5, N/2 )  
     else:
-        print('t_vec input')
-        print(tval_set)
-        t_vec = tval_set
-        
-    # now make a "t, theta" grid:
+        t_vec = tvec_set
+
     grid_t = npmat.repmat(t_vec,1,len(theta_vec))
     grid_t = np.transpose(grid_t)
-    
-    #grid_theta = np.zeros((Nrays,1))
+
     grid_theta = np.tile(theta_vec,(len(t_vec),1)).T.flatten()
     grid_theta = grid_theta.reshape(-1,1)
-    
+
     return t_vec, grid_t, grid_theta
 
 
 
+
 class Phantom:
     """ Base class for Phantoms
     """
@@ -472,13 +508,16 @@ class Phantom:
         Parameters and Attributes
         -------------------------
         phantom_type : string, can be 'ellipses', 'shepp_logan', 'modified_shepp_logan','squares','rectangles'
-        circle       : 
-        matrix       : 
+        circle       : as in the function iradon of scikit-image "assume the reconstructed image is zero outside the inscribed circle. Also changes the default output_size to match the behaviour of radon called with circle=True."
+        matrix       : matrix of the phantom
         """
         self.phantom_type = phantom_type
         self.circle = circle
         if matrix is None:
             self.matrix = my_phantomgallery(phantom_type)
+        else:
+            self.matrix = matrix
+        #endif
 
     def get_phantom( self, N = 128 ):
         """ Compute the matrix phantom image.
@@ -486,29 +525,29 @@ class Phantom:
         Parameters
         ----------
         N : number of pixels of the image in each dimension
-        
+
         Returns
         -------
         P : phantom image
         """
         if self.phantom_type in ['ellipses', 'shepp_logan', 'modified_shepp_logan']:
-            P = phantom_ellipses(N, self.matrix)
+            P = phantom_ellipses(N, np.copy(self.matrix))
         elif self.phantom_type == 'squares':
-            P = phantom_squares(N, self.matrix)
+            P = phantom_squares(N, np.copy(self.matrix))
         elif self.phantom_type == 'rectangles':
-            P = phantom_rectangles(N, self.matrix)
+            P = phantom_rectangles(N, np.copy(self.matrix))
         else:
             print('Error in the choice of the phantom')
         return P
 
     def get_sinogram( self, N = 128, theta_vec = None ):
         """ Return the Analytical Sinogram of the phantom.
-        
+
         Parameters
         ----------
         N         : number of pixels of the image in each dimension
         theta_vec : vector of the angles theta
-    
+
         Returns
         -------
         analytical_sinogram : matrix of the Analyitical Sinogram of the phantom
@@ -517,11 +556,11 @@ class Phantom:
         if theta_vec is None:
             theta_vec = np.deg2rad(np.linspace(0, 359, 360))
         if self.phantom_type in ['ellipses', 'shepp_logan', 'modified_shepp_logan']:
-                analytical_sinogram = radon_ellipses(N, theta_vec, self.matrix, circle=self.circle);
+                analytical_sinogram = radon_ellipses(N, theta_vec, np.copy(self.matrix), circle=self.circle);
         elif self.phantom_type == 'squares':
-                analytical_sinogram = radon_squares(N, theta_vec, self.matrix, circle = self.circle);
+                analytical_sinogram = radon_squares(N, theta_vec, np.copy(self.matrix), circle = self.circle);
         elif self.phantom_type == 'rectangles':
-                analytical_sinogram = radon_rectangles(N, theta_vec, self.matrix,  circle=self.circle);
+                analytical_sinogram = radon_rectangles(N, theta_vec, np.copy(self.matrix),  circle=self.circle);
         else:
             print('error on the choice of phantom type')
         return analytical_sinogram
